import traceback
from mengling_tool2.data.make import encode
from ..functions.args_get import getFakeUserAgent


class Spider:
    def __init__(self, timeout=15, headers: dict = None, proxies: str = None,
                 cookie: str = None, user_agent: str = None,
                 ifrandom=False, **kwargs):
        self.timeout = timeout
        self.headers = headers if headers is not None else dict()
        user_agent_have = self.headers.pop('user-agent', '') + self.headers.pop('User-Agent', '')
        if user_agent is None:
            if ifrandom:
                # 使用随机头
                user_agent = getFakeUserAgent()
            else:
                if len(user_agent_have) > 0:
                    user_agent = user_agent_have
                else:
                    # 使用默认头
                    user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36'
        self.headers['user-agent'] = user_agent
        if cookie is not None: self.headers['cookie'] = cookie
        self.proxies = proxies
        self.config = kwargs
        self.spider = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def get(self, url, ifobj=False, savepath=None, ifjson=False, **kwargs):
        pass

    def post(self, url, data=None, ifobj=False, savepath=None, ifjson=False, **kwargs):
        pass

    # 保存文件
    def save(self, url, result, savepath):
        try:
            with open(f'{savepath}/{encode(url)}.html', encoding='utf-8',
                      mode='w+') as file:
                file.write(result)
        except:
            traceback.print_exc()
            print(url, '\n保存文件失败!')

    # # 检查型获取
    # def checkGet(self, url, txt_checkfunc_b, **kwargs):
    #     txt = self.get(url, **kwargs)
    #     assert txt_checkfunc_b(txt), '不满足条件!'
    #     return txt

    def close(self):
        if self.spider:
            self.spider.close()



import json, time
from threading import Thread, Lock
from . import webfunc
from menglingtool.goodlib import ThreadDictGoods
from menglingtool.progress import jdprint
from menglingtool.thread import thread_auto_run
from menglingtool_redis.redis_tool import RedisExecutor
from menglingtool_spiders.functions.make import getHeaders_str
from menglingtool_spiders.functions.args_get import getFakeUserAgent
from menglingtool_spiders.spiders.httpx import Httpx

__dlfuncs__ = [
    webfunc.freeProxy01,
    webfunc.freeProxy02,
    webfunc.freeProxy03,
    webfunc.freeProxy04,
    webfunc.freeProxy05,
    webfunc.freeProxy06,
    webfunc.freeProxy07,
    webfunc.freeProxy08,
    webfunc.freeProxy10,
    webfunc.freeProxy12,
    webfunc.freeProxy14,
    webfunc.my_dl01,
    webfunc.my_dl02,
    webfunc.my_dl03,
    webfunc.qtdl,
]


def getWebIpdt():
    while True:
        for func in __dlfuncs__:
            try:
                ipdts = func()
                assert len(ipdts) > 0, f'{func} 返回数量为0!'
            except:
                # traceback.print_exc()
                continue
            for ipdt in ipdts:
                yield ipdt


ymx_headers = getHeaders_str('''
accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9
accept-encoding: gzip, deflate, br
accept-language: zh-CN,zh;q=0.9
sec-ch-ua: "Google Chrome";v="89", "Chromium";v="89", ";Not A Brand";v="99"
sec-ch-ua-mobile: ?0
sec-fetch-dest: document
sec-fetch-mode: navigate
sec-fetch-site: none
sec-fetch-user: ?1
upgrade-insecure-requests: 1
user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36
''')


class DL:
    def __init__(self, name, dbindex: int, host: str, port=6379, pwd: str = None,
                 checkurl=None, checkweek=60, maxerrornum=1, min_ip_num=20, ifcheck=True):
        self.name = name
        self.goodt = ThreadDictGoods({'spider': [Httpx, {'headers': ymx_headers}],  #
                                      'r': [RedisExecutor,
                                            {'dbindex': dbindex, 'host': host, 'port': port, 'pwd': pwd}]})
        self.checkurl = checkurl
        # self.lock = Lock()
        self.ifcheck = ifcheck
        self.checkweek = checkweek
        self.maxerrornum = maxerrornum
        self.min_ip_num = min_ip_num
        # 记录非检查下的出错ip
        self.erroripst = set()
        self._lock = Lock()

    # 非检验错误通知
    def errorInform(self, ip):
        r = self.goodt.getThreadKeyGood('r')
        ipdt = json.loads(r.hget(self.name, ip))
        ipdt['error'] = ipdt.get('error', 0) + 1
        r.hset(self.name, ip, json.dumps(ipdt))

    # 非检验成功通知
    def sucInform(self, ip):
        r = self.goodt.getThreadKeyGood('r')
        ipdt = json.loads(r.hget(self.name, ip))
        ipdt['error'] = 0
        r.hset(self.name, ip, json.dumps(ipdt))

    # 验证方法
    def yz(self, ip) -> bool:
        spider = self.goodt.getThreadKeyGood('spider')
        try:
            spider.headers['user-agent'] = getFakeUserAgent()
            txt = spider.get(self.checkurl, proxies=ip)
            assert len(txt) > 10_0000
            # assert spider.get(self.checkurl, proxies=ip,ifobj=True).status_code == 200
            return True
        except:
            return False

    # 验证已有ip
    def checkHaveIps(self):
        def opition(ipdt_txt):
            r = self.goodt.getThreadKeyGood('r')
            ipdt = json.loads(ipdt_txt)
            error = ipdt.pop('error', 0)
            ip = ipdt['host']
            b = self.yz(ip)
            with self._lock:
                if not b:
                    error += 1
                    if error > self.maxerrornum:
                        r.hdel(self.name, ip)
                        jdprint('[删除]', ip)
                    else:
                        ipdt['error'] = error
                        r.hset(self.name, ip, json.dumps(ipdt))
                        jdprint(f'[错误{error}次]', ip)
                else:
                    ipdt['error'] = 0
                    ipdt['successful'] = ipdt.get('successful', 0) + 1
                    r.hset(self.name, ip, json.dumps(ipdt))
                    jdprint(f'[校验成功]', ip)
            self.goodt.delThreadKeyGood(iftz=False)

        r = self.goodt.getThreadKeyGood('r')
        while True:
            jdprint('开始检查...')
            with self._lock:
                ipdts = r.hvals(self.name)
            if self.ifcheck:
                # 自检查
                thread_auto_run(opition, ipdts, threadnum=5, if_error=False, iftz=False)
            else:
                for ipdt_txt in ipdts:
                    ipdt = json.loads(ipdt_txt)
                    if ipdt.get('error', 0) > self.maxerrornum:
                        with self._lock:
                            r.hdel(self.name, ipdt['host'])
                        jdprint('[删除]', ipdt['host'])
                        self.erroripst.add(ipdt['host'])
            time.sleep(self.checkweek)

    # 收集新ip
    def collect(self):
        ipit = getWebIpdt()

        def opition(ipdt):
            r = self.goodt.getThreadKeyGood('r')
            ip = ipdt['host']
            with self._lock:
                b = r.hexists(self.name, ip)
            if b:
                jdprint('[已存在]', ip)
            else:
                b = self.yz(ip)
                with self._lock:
                    if b and self.ifcheck:
                        r.hset(self.name, ip, json.dumps(ipdt))
                        jdprint('[新增]', ipdt)
                    elif self.ifcheck:
                        jdprint('[失败]', ip)
                    elif ip in self.erroripst:
                        jdprint('[失败ip]', ip)
                    else:
                        r.hset(self.name, ip, json.dumps(ipdt))
                        jdprint('[非检测新增]', ipdt)
            self.goodt.delThreadKeyGood(iftz=False)

        r = self.goodt.getThreadKeyGood('r')
        while True:
            with self._lock:
                iplen = r.hlen(self.name)
            if iplen < self.min_ip_num:
                jdprint(f'数量不足,开始收集{self.min_ip_num * 2}个...')
                ipdtags = [next(ipit) for i in range(self.min_ip_num * 2)]
                thread_auto_run(opition, ipdtags, threadnum=5, if_error=False, iftz=False)
            else:
                jdprint('数量已足够,收集进入休息...')
                time.sleep(self.checkweek / 2)

    def run(self):
        t1 = Thread(target=self.checkHaveIps)
        t1.start()
        t2 = Thread(target=self.collect)
        t2.start()
        t1.join()
        t2.join()

#
# if __name__ == '__main__':
#     # 参数
#     dbindex, name, connect = 14, '测试代理池', config.REDIS_CONNECT
#     maxerrornum = 1
#     min_ip_num = 30
#     ifcheck = True
#     checkweek = 60
#     checkurl = 'https://www.amazon.com/' if ifcheck else None
#
#     # 流程
#     DL(dbindex, name, connect, checkurl, maxerrornum=maxerrornum,
#        min_ip_num=min_ip_num, ifcheck=ifcheck, checkweek=checkweek).run()



import base64
import ctypes.wintypes
import json, os, re, sys
import sqlite3
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


# 获取对应的cookie文本
def getCookieStr(drivercookies: list, *names):
    cs = []
    names = set(names)
    for cookie in drivercookies:
        if len(names) == 0 or cookie['domain'] in names:
            cs.append('%s=%s' % (cookie['name'], cookie['value']))
    return '; '.join(cs)


'''用于解密encrypted_value的配套方法'''


def __dpapi_decrypt(encrypted):
    class DATA_BLOB(ctypes.Structure):
        _fields_ = [('cbData', ctypes.wintypes.DWORD),
                    ('pbData', ctypes.POINTER(ctypes.c_char))]

    p = ctypes.create_string_buffer(encrypted, len(encrypted))
    blobin = DATA_BLOB(ctypes.sizeof(p), p)
    blobout = DATA_BLOB()
    retval = ctypes.windll.crypt32.CryptUnprotectData(
        ctypes.byref(blobin), None, None, None, None, 0, ctypes.byref(blobout))
    if not retval:
        raise ctypes.WinError()
    result = ctypes.string_at(blobout.pbData, blobout.cbData)
    ctypes.windll.kernel32.LocalFree(blobout.pbData)
    return result


def __aes_decrypt(datapath, encrypted_txt):
    with open(fr'{datapath}\Local State', encoding='utf-8',
              mode="r") as f:
        jsn = json.loads(str(f.readline()))
    encoded_key = jsn["os_crypt"]["encrypted_key"]
    encrypted_key = base64.b64decode(encoded_key.encode())
    encrypted_key = encrypted_key[5:]
    key = __dpapi_decrypt(encrypted_key)
    nonce = encrypted_txt[3:15]
    cipher = Cipher(algorithms.AES(key), None, backend=default_backend())
    cipher.mode = modes.GCM(nonce)
    decryptor = cipher.decryptor()
    return decryptor.update(encrypted_txt[15:])


def __chrome_decrypt(datapath, encrypted_txt):
    if sys.platform == 'win32':
        try:
            if encrypted_txt[:4] == b'x01x00x00x00':
                decrypted_txt = __dpapi_decrypt(encrypted_txt)
                return decrypted_txt.decode()
            elif encrypted_txt[:3] == b'v10':
                decrypted_txt = __aes_decrypt(datapath, encrypted_txt)
                return decrypted_txt[:-16].decode()
        except WindowsError:
            return None
    else:
        raise WindowsError


# 获取当地谷歌浏览器的cookies文件数据
def __getChromiumCookiedts(*domains, pattern=None, customize_cookiepath=None, datapath=None,
                           if_edge=False, ifhigh_version=False, iftest_chromium=False) -> list:
    if customize_cookiepath:
        # 使用自定义cookie位置
        cookiepath = customize_cookiepath
    else:
        if datapath is None:
            if iftest_chromium:
                __user_data_hz__ = r"\Chromium\User Data"
            elif if_edge:
                __user_data_hz__ = r"\Microsoft\Edge\User Data"
            else:
                __user_data_hz__ = r"\Google\Chrome\User Data"
            datapath = f'{os.environ["LOCALAPPDATA"]}{__user_data_hz__}'
        if ifhigh_version:
            cookiepath = fr'{datapath}\Default\Network\Cookies'
        else:
            cookiepath = fr'{datapath}\Default\Cookies'
    sql = "select host_key,name,encrypted_value from cookies"
    if len(domains) > 0 or pattern is not None:
        sql += " where "
        for domain in domains:
            sql += "host_key='%s' or " % domain
        if pattern is not None: sql += "host_key like '%s' or " % pattern
        sql = str(sql[:-4])
    cookiedts = []
    with sqlite3.connect(cookiepath) as conn:
        conn.text_factory = lambda x: str(x, 'utf-8', 'ignore')
        cu = conn.cursor()
        cls = cu.execute(sql).fetchall()
        lie0 = cu.description  # 获取列
        # 记录列名
        if lie0 is not None:
            lies = [lie[0] for lie in lie0]
        else:
            lies = []
        for ts in cls:
            cookies = dict()
            for i in range(len(ts)):
                if lies[i] == 'encrypted_value':  # 加密处理
                    cookies[lies[i]] = __chrome_decrypt(datapath, ts[i])
                    # 记录到value值
                    cookies['value'] = __chrome_decrypt(datapath, ts[i])
                elif lies[i] == 'value':  # 原value不记录
                    continue
                elif lies[i] == 'host_key':
                    cookies['domain'] = ts[i]
                else:
                    cookies[lies[i]] = ts[i]
            cookiedts.append(cookies)
    return cookiedts


# 获取谷歌浏览器cookie字典
def getGoogleCookiedts(*domains, pattern=None, datapath=None, ifhigh_version=False) -> list:
    return __getChromiumCookiedts(*domains, pattern=pattern, customize_cookiepath=None, datapath=datapath,
                                  if_edge=False, ifhigh_version=ifhigh_version, iftest_chromium=False)


# 获取edge浏览器cookie字典
def getEdgeCookiedts(*domains, pattern=None, datapath=None, ifhigh_version=True) -> list:
    return __getChromiumCookiedts(*domains, pattern=pattern, customize_cookiepath=None, datapath=datapath,
                                  if_edge=True, ifhigh_version=ifhigh_version, iftest_chromium=False)


# 获取测试浏览器cookie字典
def getTestChromiumCookiedts(*domains, pattern=None, datapath=None, ifhigh_version=False) -> list:
    return __getChromiumCookiedts(*domains, pattern=pattern, customize_cookiepath=None, datapath=datapath,
                                  if_edge=False, ifhigh_version=ifhigh_version, iftest_chromium=True)


# 获取cookie自定义路径字典
def getCustomizeCookiedts(*domains, customize_cookiepath: str, pattern=None) -> list:
    return __getChromiumCookiedts(*domains, pattern=pattern, customize_cookiepath=customize_cookiepath)


# 获取当地火狐浏览器的cookies文件数据，高版本存在获取不到的情况
def getLocalFirefoxCookieList(*domains, pattern=None, datapath=None):
    if datapath is None:
        cookiepath_common = os.environ['APPDATA'] + r"\Mozilla\Firefox\Profiles"
    else:
        cookiepath_common = datapath
    folds_arr = os.listdir(cookiepath_common)
    folds_end = [os.path.splitext(file)[-1][1:] for file in folds_arr]

    if 'default-release' in folds_end:
        cookie_fold_index = folds_end.index('default-release')
    else:
        cookie_fold_index = folds_end.index('default')
    cookie_fold = folds_arr[cookie_fold_index]
    cookie_path = os.path.join(os.path.join(cookiepath_common, cookie_fold), 'cookies.sqlite')
    # 获取cookie数据
    with sqlite3.connect(cookie_path) as conn:
        cur = conn.cursor()
        sql = "select baseDomain, name, value from moz_cookies"
        if len(domains) > 0 or pattern is not None:
            sql += " where "
            for domain in domains:
                sql += "baseDomain ='%s' or " % domain[1:]
            if pattern is not None: sql += "baseDomain like '%s' or " % pattern
            sql = str(sql[:-4])

        cookiedts = [{'domain': '.' + baseDomain, 'name': name, 'value': value}
                     for baseDomain, name, value in cur.execute(sql).fetchall()
                     if name != 'miniDialog']
    return cookiedts


def getCookie_txt(cookiedts: list, keys=None):
    alldt = dict()
    for dt in cookiedts:
        alldt[dt["name"]] = dt["value"]
    if keys is None:
        cookies = [f'{name}={value}' for name, value in alldt.items()]
    else:
        cookies = [f'{key}={alldt[key]}' for key in keys]
    cookie = '; '.join(cookies)
    return cookie


# 校验cookie提取对应字典映射表
def getCookieMapdt(cookie: str, first_host: str, cookiedts: list):
    txtdt = dict()
    for txt in cookie.split('; '):
        k, v = re.findall('([^=]+)=(.+)', txt)[0]
        txtdt[k] = v
    mapdt = dict()
    for key, value in txtdt.items():
        hs = list()
        for dt in cookiedts:
            if dt['value'] == value:
                if dt['domain'] == first_host and dt['name'] == key:
                    mapdt[key] = first_host
                    hs = 'have'
                    break
                hs.append(f'{key}->{dt}')
        if len(hs) == 0:
            print(key, value, '没有!')
        elif hs != 'have':
            [print(t) for t in hs]
    return mapdt


# 从给定字典中提取cookie
def getDetailedCookie(cookiedts: list, key_host: dict) -> str:
    cookies = list()
    for key, host in key_host.items():
        ifhave = False
        for dt in cookiedts:
            if dt['domain'] == host and dt['name'] == key:
                cookies.append(f'{key}={dt["value"]}')
                ifhave = True
                break
        if not ifhave:
            print(f'{host} -> {key} 没有找到cookie值!')
    return '; '.join(cookies)



import time
from PIL import Image
from selenium.webdriver import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
import traceback

# 操作后等待元素变化
def runWaitElement(driver, xpath, func, *args, **kwargs):
    timeout = kwargs.get('timeout', 30)
    locator = (By.XPATH, xpath)
    oldpage = WebDriverWait(driver, timeout).until(EC.presence_of_element_located(locator)).get_attribute("innerHTML")
    func(*args, **kwargs)
    # 等待时间
    for i in range(timeout):
        time.sleep(1)
        locator = (By.XPATH, xpath)
        newpage = WebDriverWait(driver, timeout).until(EC.presence_of_element_located(locator)) \
            .get_attribute("innerHTML")
        if newpage != oldpage: break


# 添加所有有效的cookies，提示无效的cookies
def addCookies(driver, cookies, iftz):
    for cookie in cookies:
        try:
            driver.add_cookie(cookie)
        except:
            if iftz:
                traceback.print_exc()
                print('[cookie无效]', cookie)
    driver.refresh()


# 模拟清空
def keyClear(e): e.clear()


# 模拟输入
def keyin(e, txt):
    e.send_keys(txt)



# 模拟点击,直接调用js,需要使用唯一xapth,否则只取第一个元素
def click(driver, xpath):
    # driver.execute_script("document.getElementById('" + id + "').onlick")
    driver.execute_script(
        f'document.evaluate(\'{xpath}\', document, null, XPathResult.ANY_TYPE, null).iterateNext().click()')


# 选择下拉栏选项
def select(e, value_text):
    s = Select(e)
    s.select_by_visible_text(value_text)  # 通过文本


# 模拟浏览器下滑至底部
def scroll(driver, maxwaitime=30, speed=500):
    driver.execute_script(""" 
        (function () { 
            window.wait=true;
            var y = document.documentElement.scrollTop; 
            var step = %s; 
            function f() { 
                if (y <= document.body.scrollHeight) { 
                    y += step; 
                    window.scroll(0, y); 
                    setTimeout(f, 50); 
                }
                else { 
                    window.scroll(0, y); 
                    window.wait=false;
                } 
            } setTimeout(f, 500); })(); """ % speed)
    for i in range(maxwaitime):
        time.sleep(1)
        if not driver.execute_script('return window.wait;'): break


# 跳至浏览器底部
def goDown(driver):
    driver.execute_script(""" 
            (function () { 
                var y = document.body.scrollHeight; 
                window.scroll(0, y); 
                })(); """)


# 跳至浏览器顶部
def goUp(driver):
    driver.execute_script(""" 
            (function () { 
                var y = document.body.scrollHeight; 
                window.scroll(0, -y); 
                })(); """)


# 移除所有指定标签
def removeAllTags(driver, *tagnames):
    for tagname in tagnames:
        # 移除所有script
        driver.execute_script('''
                    var ss=document.getElementsByTagName("%s");
                    for(var i = ss.length-1; i >=0; i--){
                        ss[i].remove()}''' % tagname)


# 跳转嵌套页面
def changeFrame(driver, iframe_e):
    driver.switch_to.frame(iframe_e)  # 切换到iframe


# 跳转至最外层页面
def parentFrame(driver):
    driver.switch_to.default_content()


# 截图
def screenshot(driver, filepath):
    driver.get_screenshot_as_file(filepath)


# 获取验证码值
def saveYZM(driver, img_e, filepath='temp.png'):
    # 对验证码所在位置进行定位，然后截取验证码图片
    location = img_e.location
    size = img_e.size
    left = location['x']
    top = location['y']
    right = left + size['width']
    bottom = top + size['height']
    driver.save_screenshot(filepath)
    image_obj = Image.open(filepath).crop((left, top, right, bottom))
    image_obj.save(filepath)
    return filepath


def slideDragAction(e, x):
    # 找到滑块
    ActionChains(e).drag_and_drop_by_offset(e, x, 0).perform()


# 清空sessionStorage字典
def clearSessionStorage(driver):
    driver.execute_script("window.sessionStorage.clear();")


# 添加数据至sessionStorage字典
def addSessionStorage(driver, dt):
    for key in dt.keys():
        driver.execute_script("window.sessionStorage.setItem(arguments[0], arguments[1]);", key, dt[key])



import socket
import time, os
import requests

CRO_headers = {
    'user-agent':
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36'
}


# 下载链接至本地
def downData(url, path, filefullname, headers=None, timeout=30, proxies=None, ifcheck_have=True):
    global CRO_headers
    if not headers:
        headers = CRO_headers
    elif 'user-agent' not in headers.keys() and 'User-Agent' not in headers.keys():
        headers['user-agent'] = CRO_headers['user-agent']
    if timeout > 0: socket.setdefaulttimeout(timeout)  # 解决下载不完全问题且避免陷入死循环
    filepath = f'{path}/{filefullname}'
    try:
        # 查看路径是否存在
        if not os.path.exists(path):  os.makedirs(path)
        if ifcheck_have and os.path.exists(filepath):
            print('已存在: ', filepath)
        else:
            proxies = {'https': f'http://{proxies}', 'http': f'http://{proxies}'} if proxies else None
            req = requests.get(url, proxies=proxies, headers=headers)
            with open(filepath, mode='wb') as file:
                file.write(req.content)
    except socket.timeout:
        print(url)
        raise socket.timeout
    except Exception as e:
        print(url)
        raise e



from scrapy import Selector


# 获取解析对象
def getSelector(txt: str) -> Selector:
    return Selector(text=txt)


# 获取元素下全部文本,排除非空
def getAllTexts(e: Selector, e_xpath: str, if_remove_null=True) -> list:
    txts = e.xpath(e_xpath + '//text()').extract()
    return [txt.strip() for txt in txts if not if_remove_null or len(txt.strip()) > 0]



import asyncio
from .__pyppeteer__.action import Body, Act
from .__pyppeteer__.request import Req


def _run(task):
    asyncio.get_event_loop().run_until_complete(asyncio.gather(task))


class PyppTool(Body, Act, Req):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        _run(self.initBrowser())

    def close(self):
        _run(self.browser.close())


# 每个任务对应一个page
def tasksRun(*tasks):
    async def main():
        for i, task in enumerate(tasks):
            task = asyncio.create_task(task(), name=f'task-{i}')
            await task

    asyncio.run(main())



import json
import traceback
import urllib.parse
import execjs
import requests


# 谷歌翻译，英翻汉
def translate(text):
    assert len(text) < 4891, "翻译的长度超过限制！！！"
    tk = execjs.compile("""
           function TL(a) {
           var k = "";
           var b = 406644;
           var b1 = 3293161072;
           var jd = ".";
           var $b = "+-a^+6";
           var Zb = "+-3^+b+-f";

           for (var e = [], f = 0, g = 0; g < a.length; g++) {
               var m = a.charCodeAt(g);
               128 > m ? e[f++] = m : (2048 > m ? e[f++] = m >> 6 | 192 : (55296 == (m & 64512) && g + 1 < a.length && 56320 == (a.charCodeAt(g + 1) & 64512) ? (m = 65536 + ((m & 1023) << 10) + (a.charCodeAt(++g) & 1023),
               e[f++] = m >> 18 | 240,
               e[f++] = m >> 12 & 63 | 128) : e[f++] = m >> 12 | 224,
               e[f++] = m >> 6 & 63 | 128),
               e[f++] = m & 63 | 128)
           }
           a = b;
           for (f = 0; f < e.length; f++) a += e[f],
           a = RL(a, $b);
           a = RL(a, Zb);
           a ^= b1 || 0;
           0 > a && (a = (a & 2147483647) + 2147483648);
           a %= 1E6;
           return a.toString() + jd + (a ^ b)
       };

       function RL(a, b) {
           var t = "a";
           var Yb = "+";
           for (var c = 0; c < b.length - 2; c += 3) {
               var d = b.charAt(c + 2),
               d = d >= t ? d.charCodeAt(0) - 87 : Number(d),
               d = b.charAt(c + 1) == Yb ? a >>> d: a << d;
               a = b.charAt(c) == Yb ? a + d & 4294967295 : a ^ d
           }
           return a
       }
       """).call("TL", text)

    content = urllib.parse.quote(text)
    url = "http://translate.google.cn/translate_a/single?client=t" \
          "&sl=en&tl=zh-CN&hl=zh-CN&dt=at&dt=bd&dt=ex&dt=ld&dt=md&dt=qca" \
          "&dt=rw&dt=rm&dt=ss&dt=t&ie=UTF-8&oe=UTF-8&clearbtn=1&otf=1&pc=1" \
          "&srcrom=0&ssel=0&tsel=0&kc=2&tk=%s&q=%s" % (tk, content)
    result = requests.get(url).text
    end = result.find("\",")
    if end > 4: result = result[4:end]
    return result


# 获取识别后的验证码
def getYZM(base64):
    # 调用接口来进行识别,非本地ocr软件
    try:
        js = json.loads(requests.post(
            'https://www.paddlepaddle.org.cn/paddlehub-api/image_classification/chinese_ocr_db_crnn_mobile',
            data='{"image":"%s"}' % base64).text)
        result = js['result'][0]['data'][0]['text']
        return result
    except:
        traceback.print_exc()
        print('[验证码解析失败]', js['result'][0])
        return None



import re
import socket
from scrapy import Selector
from urllib.parse import urlparse, urlencode, quote, unquote


# 设置获取数据类型
def getDatas(yuan):
    # 获取所有链接
    pattern = '(https?://[^\s)";]+(\.(\w|/)*))'
    link = re.compile(pattern).findall(yuan)
    # 将文件链接去重并统一输出为列表格式
    return list(set([li[0] for li in link]))


def getTable(page_source, table_xpath, lie_xpath, row_xpath) -> list:
    table = Selector(text=page_source).xpath(table_xpath)
    lies = []
    for lie in table.xpath(lie_xpath):
        lies.append(lie.xpath('string(.)').get().replace('\n', '').strip())
    rowdts = []
    for row in table.xpath(row_xpath):
        rowdt = dict((lie, td.xpath('string(.)').get().replace('\n', '').strip())
                     for lie, td in zip(lies, row.xpath('./td')) if lie)
        rowdts.append(rowdt)
    return rowdts


#
# # 获取表格字典,列与值同级
# def geTableDt_thtr(table_lxml) -> dict:
#     datadt = dict()
#     for tr in table_lxml.xpath('./tbody/tr'):
#         lie = tr.xpath('./th')[0].xpath('string(.)').strip()
#         value = tr.xpath('./td')[0].xpath('string(.)').strip()
#         datadt[lie] = value
#     return datadt
#
#
# def geTableDt_lies(table_lxml, lies, minindex=0, maxindex=None):
#     datadts = list()
#     trs = table_lxml.xpath('./tbody/tr')
#     trs = trs[minindex:] if maxindex is None else trs[minindex:maxindex]
#     for tr in trs:
#         dt = dict()
#         for lie, td in zip(lies, tr.xpath('./td')):
#             value = td.xpath('string(.)').strip()
#             dt[lie] = value
#         datadts.append(dt)
#     return datadts
#
#
# # 获取表格字典
# def geTableDt(table_lxml) -> list:
#     datadts = list()
#     lies = list()
#     trs = table_lxml.xpath('./tbody/tr')
#     for th in trs[0].xpath('./th'):
#         lies.append(th.xpath('string(.)').strip())
#     for tr in trs[1:]:
#         dt = dict()
#         for lie, td in zip(lies, tr.xpath('./td')):
#             value = td.xpath('string(.)').strip()
#             dt[lie] = value
#         datadts.append(dt)
#     return datadts


# 获取嵌入后的有效链接，替代直接用format
def getUrlFormat(url0, **valuedt):
    for key in valuedt:
        # 转义为url字段，避免类似&的情况
        valuedt[key] = quote(str(valuedt[key]))
    return url0.format(**valuedt)


# 获取参数编码后的链接
def getEncodeUrl(url0, valuedt: dict):
    encode_params = urlencode(list(valuedt.items()))
    return f'{url0}?{encode_params}'


# 获取url编码文本
def getUrlEncodeStr(txt: str, encoding='utf-8') -> str:
    return quote(txt, encoding=encoding)


# 获取url解码文本
def getUrlDecodeStr(txt) -> str:
    return unquote(txt)


# 获取链接的参数字典
def getUrlKwargs(txt) -> (str, dict):
    dt = dict()
    for t in re.findall('([^?=&]+=[^?=&]+)', txt):
        k, v = t.split('=')
        dt[unquote(k)] = unquote(v)
    return unquote(txt.split('?')[0]), dt


# 字符串转headers
def getHeaders_str(string) -> dict:
    headers = dict()
    lines = string.split('\n')
    for line in lines:
        line = line.strip()
        if line != '':
            key, value = str(re.match('.+?:', line).group()[0:-1]), str(re.search(':.+', line).group()[1:])
            headers[key] = value.strip()
    return headers


# 超时报错方法
def sockeTimeoutRun(timeout, func, *args, **kwargs):
    try:
        socket.setdefaulttimeout(timeout)
        return func(*args, **kwargs)
    except socket.timeout:
        raise socket.timeout


# 获取连接后的新链接
def getNewUrl(url, **kwargs):
    parsed_url = urlparse(url)
    query = urlencode(kwargs)
    new_url = parsed_url._replace(query=f'{parsed_url.query}&{query}').geturl()
    return new_url



from pyppeteer import launch
import asyncio


class Body:
    def __init__(self, executablePath=None, datapath=None, proxies=None,
                 headless=False, minload=False,
                 is_iphone=False,
                 ):
        self.browser = None
        self._map_pagers = {}
        self.executablePath = executablePath
        self.datapath = datapath
        self.proxies = proxies
        # self.max_window = max_window
        self.headless = headless
        self.minload = minload
        self.is_iphone = is_iphone

    def getTaskPagers(self):
        task_name = asyncio.current_task().get_name()
        if self._map_pagers.get(task_name) is None:
            self._map_pagers[task_name] = []
        return self._map_pagers[task_name]

    async def _setPage(self, *pagers):
        # if min_window: ()  # 浏览器窗口最小化
        # 防止反爬虫检测
        # 以下为插入中间js，将淘宝会为了检测浏览器而调用的js修改其结果。
        for pager in pagers:
            await pager.evaluateOnNewDocument(
                '() =>{Object.defineProperties(navigator,{webdriver:{get: () => undefined}})}')
            # 设置窗口尺寸
            await pager.setViewport({'width': 1920, 'height': 1080})
            # 模拟手机端
            if self.is_iphone:
                await pager.setUserAgent(
                    "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB200 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1")
            else:
                await pager.setUserAgent(
                    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.75 Safari/537.36")

    async def initBrowser(self):
        arglist = ['--enable-automation', '--no-sandbox', '--log-level=30']
        if self.datapath: arglist.append(f'--user-data-dir={self.datapath}')
        if self.proxies: arglist.append('--proxy-server=' + self.proxies)  # 使用代理
        arglist.append("--window-size=1960,1080")
        # 最低资源加载,图片禁用暂不生效
        if self.minload:
            arglist.append('--disable-gpu')
            arglist.append("--disable-javascript")
            arglist.append('blink-settings=imagesEnabled=false')
            arglist.append('--profile.managed_default_content_settings.images=2')
        # 语言默认英文
        # arglist.append('--lang=en_US')
        # 忽略私密链接
        arglist.append('--ignore-certificate-errors')
        # 关闭自动化提示框
        arglist.append('--disable-infobars')
        arglist.append('--permissions.default.stylesheet=2')
        self.browser = await launch(executablePath=self.executablePath, headless=self.headless,
                                    autoClose=False, dumpio=True, args=arglist)
        pagers = await self.browser.pages()
        self._map_pagers['task-0'] = pagers
        await self._setPage(*pagers)

    async def newWindow(self):
        pagers = self.getTaskPagers()
        pager = await self.browser.newPage()
        await self._setPage(pager)
        pagers.append(pager)



import requests
from .__superclass__ import Spider
import json

class Requests(Spider):
    def __init__(self, **kwargs):
        Spider.__init__(self, **kwargs)
        self.proxies_func = lambda p: {'https': f'http://{p}', 'http': f'http://{p}'}

    def head(self, url, ifobj=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            r = requests.head(url, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            r = requests.head(url, **kwargs)
        if ifobj:
            return r
        else:
            return r.status_code

    def get(self, url, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            r = requests.get(url, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            r = requests.get(url, **kwargs)
        # 保存源代码
        if savepath is not None: self.save(url, r.text, savepath)
        if ifobj:
            return r
        else:
            txt = r.text
            if ifjson:
                return json.loads(txt)
            else:
                return txt

    def post(self, url, data=None, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            r = requests.post(url, data, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            r = requests.post(url, data, **kwargs)
        # 保存源代码
        if savepath is not None: self.save(url, r.text, savepath)
        if ifobj:
            return r
        else:
            txt = r.text
            if ifjson:
                return json.loads(txt)
            else:
                return txt



import time
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait


# 等待获取元素加载
def getWaitElement(self, constraint, constraint_class='xpath', timeout=10, onetime=0.2,
                   ifassert=True) -> WebElement:
    if constraint_class == 'xpath':
        constraint_class = By.XPATH
    elif constraint_class == 'id':
        constraint_class = By.ID
    elif constraint_class == 'class':
        constraint_class = By.CLASS_NAME
    elif constraint_class == 'tag':
        constraint_class = By.TAG_NAME
    else:
        assert False, 'constraint_class 约束类型出错，该元素等待方法无效！'
    if ifassert:
        e = WebDriverWait(self, timeout, onetime).until(
            EC.presence_of_element_located((constraint_class, constraint)))
    else:
        try:
            e = WebDriverWait(self, timeout, onetime).until(
                EC.presence_of_element_located((constraint_class, constraint)))
        except:
            e = None
    # time.sleep(0.5)
    return e


# 等待获取全部元素加载
##此方法不建议直接使用用于获取元素,获取元素最好用lxml获取
def getWaitElements(self, constraint, constraint_class='xpath', minnum=1, timeout=15, onetime=0.5) -> list:
    if constraint_class == 'xpath':
        constraint_class = By.XPATH
    elif constraint_class == 'id':
        constraint_class = By.ID
    elif constraint_class == 'class':
        constraint_class = By.CLASS_NAME
    elif constraint_class == 'tag':
        constraint_class = By.TAG_NAME
    else:
        assert False, 'constraint_class 约束类型出错，该元素等待方法无效！'
    es = []
    nowtimeout = timeout
    while len(es) < minnum:
        assert nowtimeout >= 0, '等待时间过长，仅发现%s个元素' % len(es)
        time.sleep(onetime)
        es = WebDriverWait(self, timeout, onetime).until(
            EC.presence_of_all_elements_located((constraint_class, constraint)))
        nowtimeout -= onetime
    return es



import httpx
from .__superclass__ import Spider
import json


class Httpx(Spider):
    def __init__(self, http2=False, **kwargs):
        Spider.__init__(self, **kwargs)
        self.http2 = http2
        # 高级设置会出现代理不生效的情况
        self.proxies_func = lambda p: f"http://{p}" if p is not None else None
        self.spider = self.__getH__(self.proxies)

    def __getH__(self, proxies):
        self.proxies = proxies
        # 装载时调用
        proxies = self.proxies_func(proxies)
        return httpx.Client(proxies=proxies, http2=self.http2)

    def head(self, url, ifobj=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.pop('proxies', self.proxies)
        kwargs['proxies'] = self.proxies_func(proxies)
        if proxies == self.proxies:
            r = httpx.head(url, **kwargs)
        else:
            self.close()
            self.spider = self.__getH__(proxies)
            r = httpx.head(url, **kwargs)
        if ifobj:
            return r
        else:
            return r.status_code

    def get(self, url, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.pop('proxies', self.proxies)
        if proxies == self.proxies:
            # 不支持socks代理方式
            r = self.spider.get(url, **kwargs)
        else:
            self.close()
            self.spider = self.__getH__(proxies)
            r = self.spider.get(url, **kwargs)
        # 保存源代码
        if savepath is not None: self.save(url, r.text, savepath)
        if ifobj:
            return r
        else:
            if ifjson:
                return json.loads(r.text)
            else:
                return r.text

    def post(self, url, data=None, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.pop('proxies', self.proxies)
        if proxies == self.proxies:
            r = self.spider.post(url, data=data, **kwargs)
        else:
            self.close()
            self.spider = self.__getH__(proxies)
            r = self.spider.post(url, data=data, **kwargs)
        # 保存源代码
        if savepath is not None: self.save(url, r.text, savepath)
        if ifobj:
            return r
        else:
            if ifjson:
                return json.loads(r.text)
            else:
                return r.text



import re, time
import base64
import requests
from scrapy import Selector

__proxies__ = {'https': 'http://172.20.26.113:4010', 'http': 'http://172.20.26.113:4010'}


def freeProxy01():
    """
    米扑代理 https://proxy.mimvp.com/
    :return:
    """
    url_list = [
        'https://proxy.mimvp.com/freeopen?proxy=in_hp',
        'https://proxy.mimvp.com/freeopen?proxy=out_hp'
    ]
    port_img_map = {'DMxMjg': '3128', 'Dgw': '80', 'DgwODA': '8080',
                    'DgwOA': '808', 'DgwMDA': '8000', 'Dg4ODg': '8888',
                    'DgwODE': '8081', 'Dk5OTk': '9999'}
    dts = []
    for url in url_list:
        html_tree = Selector(text=requests.get(url).text)
        for tr in html_tree.xpath(".//table[@class='mimvp-tbl free-proxylist-tbl']/tbody/tr"):
            try:
                ip = ''.join(tr.xpath('./td[2]/text()').extract())
                port_img = ''.join(tr.xpath('./td[3]/img/@src').extract()).split("port=")[-1]
                port = port_img_map.get(port_img[14:].replace('O0O', ''))
                if port:
                    dts.append({'host': '%s:%s' % (ip, port)})
            except Exception as e:
                pass
    return dts


def freeProxy02():
    """
    代理66 http://www.66ip.cn/
    :return:
    """
    url = "http://www.66ip.cn/mo.php"

    resp = requests.get(url, timeout=10)
    proxies = re.findall(r'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:\d{1,5})', resp.text)
    return [{'host': proxy} for proxy in proxies]


def freeProxy03():
    """
    pzzqz https://pzzqz.com/
    """
    from requests import Session
    from lxml import etree
    session = Session()
    dts = list()
    try:
        index_resp = session.get("https://pzzqz.com/", timeout=20, verify=False).text
        x_csrf_token = re.findall('X-CSRFToken": "(.*?)"', index_resp)
        if x_csrf_token:
            data = {"http": "on", "ping": "3000", "country": "cn", "ports": ""}
            proxy_resp = session.post("https://pzzqz.com/", verify=False,
                                      headers={"X-CSRFToken": x_csrf_token[0]}, json=data).json()
            tree = etree.HTML(proxy_resp["proxy_html"])
            for tr in tree.xpath("//tr"):
                ip = "".join(tr.xpath("./td[1]/text()"))
                port = "".join(tr.xpath("./td[2]/text()"))
                dts.append({'host': "%s:%s" % (ip, port)})
    except Exception as e:
        pass
    return dts


def freeProxy04():
    """
    神鸡代理 http://www.shenjidaili.com/
    :return:
    """
    url = "http://www.shenjidaili.com/product/open/"
    tree = Selector(text=requests.get(url).text)
    dts = []
    for table in tree.xpath("//table[@class='table table-hover text-white text-center table-borderless']"):
        for tr in table.xpath("./tr")[1:]:
            proxy = ''.join(tr.xpath("./td[1]/text()").extract())
            dts.append({'host': proxy.strip()})
    return dts


def freeProxy05(page_count=1):
    """
    快代理 https://www.kuaidaili.com
    """
    url_pattern = [
        'https://www.kuaidaili.com/free/inha/{}/',
    ]
    url_list = []
    for page_index in range(1, page_count + 1):
        for pattern in url_pattern:
            url_list.append(pattern.format(page_index))
    dts = []
    for url in url_list:
        tree = Selector(text=requests.get(url).text)
        proxy_list = tree.xpath('.//table//tr')
        time.sleep(1)  # 必须sleep 不然第二条请求不到数据
        for tr in proxy_list[1:]:
            dts.append({'host': ':'.join(tr.xpath('./td/text()').extract()[0:2])})
    return dts


def freeProxy06(page=2):
    """
    极速代理 https://www.superfastip.com/
    :return:
    """
    url = "https://api.superfastip.com/ip/freeip?page={page}"
    dts = []
    for i in range(page):
        page_url = url.format(page=i + 1)
        try:
            resp_json = requests.get(page_url).json()
            for each in resp_json.get("freeips", []):
                dts.append({'host': "%s:%s" % (each.get("ip", ""), each.get("port", ""))})
        except Exception as e:
            pass
    return dts


def freeProxy07():
    """
    云代理 http://www.ip3366.net/free/
    :return:
    """
    urls = ['http://www.ip3366.net/free/?stype=1', ]
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10)
        proxies = re.findall(r'<td>(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})</td>[\s\S]*?<td>(\d+)</td>', r.text)
        for proxy in proxies:
            dts.append({'host': ":".join(proxy)})
    return dts


def freeProxy08():
    """
    小幻代理 https://ip.ihuan.me/
    :return:
    """
    urls = [
        'https://ip.ihuan.me/address/5Lit5Zu9.html',
        'https://ip.ihuan.me/address/576O5Zu9.html',
        'https://ip.ihuan.me/address/6aaZ5riv.html'
    ]
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10)
        proxies = re.findall(r'>\s*?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s*?</a></td><td>(\d+)</td>',
                             r.text)
        for proxy in proxies:
            dts.append({'host': ":".join(proxy)})
    return dts


def freeProxy10():
    """
    墙外网站 cn-proxy
    :return:
    """
    urls = ['http://cn-proxy.com/']
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10, proxies=__proxies__, headers={
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36'})
        proxies = re.findall(r'<td>(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})</td>[\w\W]<td>(\d+)</td>', r.text)
        for proxy in proxies:
            dts.append({'host': ':'.join(proxy)})
    return dts


def freeProxy12():
    urls = ['https://list.proxylistplus.com/Fresh-HTTP-Proxy-List-1']
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10, proxies=__proxies__, headers={
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36'})
        proxies = re.findall(r'<td>(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})</td>[\s\S]*?<td>(\d+)</td>', r.text)
        for proxy in proxies:
            dts.append({'host': ':'.join(proxy)})
    return dts


def freeProxy13(max_page=2):
    """
    http://www.89ip.cn/index.html
    89免费代理
    :param max_page:
    :return:
    """
    base_url = 'http://www.89ip.cn/index_{}.html'
    dts = []
    for page in range(1, max_page + 1):
        url = base_url.format(page)
        r = requests.get(url, timeout=10)
        proxies = re.findall(
            r'<td.*?>[\s\S]*?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})[\s\S]*?</td>[\s\S]*?<td.*?>[\s\S]*?(\d+)[\s\S]*?</td>',
            r.text)
        for proxy in proxies:
            dts.append({'host': ':'.join(proxy)})
    return dts


def freeProxy14():
    """
    http://www.xiladaili.com/
    西拉代理
    :return:
    """
    urls = ['http://www.xiladaili.com/']
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10)
        ips = re.findall(r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:\d{1,5}", r.text)
        for ip in ips:
            dts.append({'host': ip.strip()})
    return dts


def my_dl01():
    """
    https://hidemy.name/en/proxy-list/#list
    国外代理
    :return:
    """
    urls = ['https://hidemy.name/en/proxy-list/#list',
            'https://hidemy.name/en/proxy-list/?start=64#list',
            'https://hidemy.name/en/proxy-list/?start=128#list',
            'https://hidemy.name/en/proxy-list/?start=192#list',
            'https://hidemy.name/en/proxy-list/?start=256#list',
            ]
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10, proxies=__proxies__, headers={
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36'})
        for tr in Selector(text=r.text).xpath('//tbody/tr'):
            if 'HTTP' in tr.xpath('./td[5]').xpath('string(.)').get(''):
                dts.append(
                    {'host': f'{tr.xpath("./td[1]/text()").get("1.1.1.1")}:{tr.xpath("./td[2]/text()").get(888)}'})
    return dts

    # proxies = re.findall(
    #     r'<td.*?>[\s\S]*?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})[\s\S]*?</td>[\s\S]*?<td.*?>[\s\S]*?(\d+)[\s\S]*?</td>',
    #     r.text)


def my_dl02():
    """
    http://free-proxy.cz/zh/proxylist/country/US/http/ping/all
    国外代理
    :return:
    """
    urls = ['http://free-proxy.cz/zh/proxylist/country/US/http/ping/all',
            'http://free-proxy.cz/zh/proxylist/country/US/https/ping/all',
            ]
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10, proxies=__proxies__, headers={
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36'})
        proxies = re.findall(
            'Base64\.decode\("([a-zA-Z0-9=]+)"\)\)</script></td><td style=""><span class="fport" style=\'\'>(\d+)</span>'
            , r.text)
        for code, port in proxies:
            ip = base64.b64decode(code).decode()
            dts.append({'host': f'{ip}:{port}'})
    return dts


def my_dl03():
    """
    https://ip.ihuan.me/
    国外代理
    :return:
    """
    urls = ['https://ip.ihuan.me/', ]
    dts = []
    for url in urls:
        r = requests.get(url, timeout=10, proxies=__proxies__, verify=False, headers={
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36'})
        proxies = re.findall(
            '>([0-9.]+)</a></td><td>(\d+)</td>'
            , r.text)
        for proxy in proxies:
            dts.append({'host': ':'.join(proxy)})
    return dts


# 蜻蜓代理
def qtdl():
    txt = requests.get(
        'https://proxyapi.horocn.com/api/v2/proxies?order_id=EFZP1704226103117766&num=10&format=text&line_separator=unix&can_repeat=no&user_token=6b167963764d620d1e259982f0a48a0d').text
    print(txt)
    ips = txt.split('\n')
    if len(ips) == 10:
        dts = [{'host': ip} for ip in ips]
    else:
        time.sleep(5)
        assert False, txt
    return dts


if __name__ == '__main__':
    dts = qtdl()
    [print(dt) for dt in dts]



import requests_html
from .__superclass__ import Spider
import json

# 含有渲染功能
# requests_html支持异步请求
class HttpSession(Spider):
    def __init__(self, ifrender=False, **kwargs):
        Spider.__init__(self, **kwargs)
        self.spider = requests_html.HTMLSession()
        # 是否输出为渲染后的结果
        self.ifrender = ifrender
        self.proxies_func = lambda p: {'https': f'http://{p}', 'http': f'http://{p}'}

    def get(self, url, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            obj = self.spider.get(url, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            obj = self.spider.get(url, **kwargs)
        # 渲染
        if self.ifrender:  obj.html.render()
        # 保存源代码
        if savepath is not None: self.save(url, obj.html.html, savepath)
        # 原始对象
        if ifobj:
            return obj
        else:
            txt = obj.text
            if ifjson:
                return json.loads(txt)
            else:
                return txt

    def post(self, url, data=None, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            obj = self.spider.post(url, data, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            obj = self.spider.post(url, data, **kwargs)
        # 渲染
        if self.ifrender:  obj.html.render()
        # 保存源代码
        if savepath is not None: self.save(url, obj.html.html, savepath)
        # 原始对象
        if ifobj:
            return obj
        else:
            txt = obj.text
            if ifjson:
                return json.loads(txt)
            else:
                return txt



from selenium import webdriver
from selenium.webdriver.chrome.service import Service as Chrome_Service
from selenium.webdriver.firefox.service import Service as Firefox_Service
from .stealth_min_js import driver_js


def __getGeneralOptions(options, headless, minload, proxies):
    # 设置代理
    if proxies is not None: options.add_argument(('--proxy-server=' + proxies))
    if headless: options.add_argument("--headless")
    # 最低资源加载
    if minload:
        # 禁止加载img及css
        try:
            prefs = {"profile.managed_default_content_settings.images": 2,
                     'permissions.default.stylesheet': 2}
            options.add_experimental_option("prefs", prefs)
        except:
            options.set_preference('permissions.default.image', 2)
        options.add_argument('--disable-gpu')
    else:
        try:
            prefs = {"profile.managed_default_content_settings.images": 1,
                     'permissions.default.stylesheet': 1}
            options.add_experimental_option("prefs", prefs)
        except:
            options.set_preference('permissions.default.image', 1)
    # 忽略私密链接
    options.add_argument('--ignore-certificate-errors')


def __getGeneralAction(driver, min_window, max_window, url, timeout):
    if min_window:
        driver.minimize_window()  # 浏览器窗口最小化
    elif max_window:
        driver.set_window_size(1920, 1080)
        driver.maximize_window()  # 浏览器窗口最大化
    # 设置超时时间,超时后网页还是没有加载完成则抛出异常
    driver.set_page_load_timeout(timeout)
    # 需要先进一次网页非开发人员模式才能生效
    if url: driver.get(url)
    # 增加自身句柄列表
    driver.hands = list(driver.window_handles)


def chrome_init(driver, headless, ifiphone, user_agent, minload, ifchromium, custom_cd_cm,
                proxies, getnum, min_window, max_window, url, datapath, chrome_version, timeout):
    # driver.timeout = timeout
    if custom_cd_cm is None:
        if ifchromium:
            # chromedriver的绝对路径
            driver.chromedriver_path = f"D:/python39/chromium{chrome_version}/my_cdr.exe"
            driver.chrome_path = f'D:/python39/chromium{chrome_version}/chrome.exe'
        else:
            # chromedriver的绝对路径
            driver.chromedriver_path = "D:/python39/my_cdr.exe"
            driver.chrome_path = None  # 则使用默认的谷歌浏览器
    else:
        driver.chromedriver_path, driver.chrome_path = custom_cd_cm
    # 最大打开数量,避免缓存过多
    driver.getnum = getnum

    options = webdriver.ChromeOptions()
    if datapath is not None: options.add_argument(r'--user-data-dir=' + datapath)
    # 模拟手机端
    if ifiphone:
        user_agent = 'MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB200 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1'
    else:
        if user_agent is None:
            # 避免使用headless的头文件,可能通过判断代理头不一致的方式反爬
            user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.105 Safari/537.36'
            # user_agent = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.164 Safari/537.36'
    options.add_argument('user-agent=' + user_agent)
    __getGeneralOptions(options, headless=headless, minload=minload, proxies=proxies)
    # 初始化一个driver，并且指定chromedriver的路径
    if driver.chrome_path is not None: options.binary_location = driver.chrome_path

    # 设置打开模式为非开发人员模式,2021最新方式,多页面依然有效,且屏蔽上方提示栏
    options.add_experimental_option(
        "excludeSwitches", ["enable-automation"])
    options.add_experimental_option('useAutomationExtension', False)
    options.add_argument("disable-blink-features=AutomationControlled")

    service = Chrome_Service(executable_path=driver.chromedriver_path)
    webdriver.Chrome.__init__(driver, service=service, options=options)
    # 使用特殊js改变行为参数
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": driver_js
    })
    # 设置打开模式为非开发人员模式,需要先进一次网页才能生效
    # undefined
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """Object.defineProperty(navigator, 'webdriver', {
                                  get: () => undefined
                                })"""})
    # 避免无头
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """Object.defineProperty(navigator, 'chrome', {
                                 get: () => undefined
                               })"""})
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """Object.defineProperty(navigator, 'vendor', {
                                      get: () => ''
                                    })"""})
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """Object.defineProperty(navigator, 'plugins', {
                                 get: () => [1, 2, 3],
                               })"""})
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """Object.defineProperty(navigator, 'languages', {
                                 get: () => ['zh-CN', 'zh'],
                               })"""})
    driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        "source": """Object.defineProperty(navigator, 'deviceMemory', {
                                 get: () => 8
                               })"""})
    __getGeneralAction(driver, min_window=min_window, max_window=max_window, url=url, timeout=timeout)


def firefox_init(driver, headless, ifiphone, user_agent, minload,
                 proxies, getnum, min_window, max_window, url, datapath):
    # 最大打开数量,避免缓存过多
    driver.getnum = getnum
    options = webdriver.FirefoxOptions()
    __getGeneralOptions(options, headless=headless, minload=minload, proxies=proxies)
    # 模拟手机端
    if ifiphone:
        user_agent = 'MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB200 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1'
    else:
        if user_agent is None:
            # 避免使用headless的头文件,可能通过判断代理头不一致的方式反爬
            user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:79.0) Gecko/20100101 Firefox/50.0'
            # user_agent = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.164 Safari/537.36'
    # datapath无法生效
    profile = webdriver.FirefoxProfile(datapath)
    profile.set_preference("dom.webdriver.enabled", False)
    profile.set_preference("general.useragent.override", user_agent)
    service = Firefox_Service(executable_path=r'D:\python39\geckodriver.exe')
    webdriver.Firefox.__init__(driver, service=service,
                               options=options)
    __getGeneralAction(driver, min_window=min_window, max_window=max_window, url=url, timeout=60)



import re
import socket
import requests
from anole import UserAgent

UA = None


# 获取地址对应的经纬度坐标
def getMap(addr) -> dict:
    url = "http://api.map.baidu.com/geocoding/v3/?"  # 百度地图API接口
    para = {
        "address": addr,  # 传入地址参数
        "output": "json",
        "ak": "myv6ZSUbM7bAdN3lKt4wGteYZA3noEZi"  # 百度地图开放平台申请ak
    }
    req = requests.get(url, para)
    req = req.json()
    return req['result']['location']


# 获取自身ip
def getMyIp():
    return re.search('\d+\.\d+\.\d+\.\d+', requests.get('http://pv.sohu.com/cityjson').text).group()


# 获得随机头
def getFakeUserAgent():
    global UA
    if UA is None: UA = UserAgent()
    return str(UA.random)


# 获取本机ip
def getLocalIp():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()

    return ip



from .base import Body
import asyncio


class Act(Body):
    # 切换当前显示窗口
    async def taskHandoff(self, index):
        pagers = self.getTaskPagers()
        pager = pagers[index]
        await pager.bringToFront()

    async def newGet(self, url, task_index=0, timeout=30, waituntil='load'):
        pager = self.getTaskPagers()[task_index]
        await pager.evaluate('document.getElementsByTagName("html")[0].remove()')
        await pager.goto(url, options={'waitUntil': waituntil, 'timeout': timeout * 1000})

    async def close_page(self, task_index=0):
        pagers = self.getTaskPagers()
        pager = pagers[task_index]
        await pager.close()
        pagers.remove(pager)

    # 添加所有有效的cookies，提示无效的cookies
    async def addCookies(self, cookies, task_index=0, iftz=True):
        pager = self.getTaskPagers()[task_index]
        for cookie in cookies:
            try:
                await pager.setCookie(cookie)
            except:
                if iftz: print('[cookie无效]', cookie)

    async def screenshot(self, filepath, task_index=0):
        pager = self.getTaskPagers()[task_index]
        await pager.screenshot({'path': filepath})

    # 跳转至底部
    async def jumpBottom(self, task_index=0):
        pager = self.getTaskPagers()[task_index]
        await pager.evaluate('window.scrollBy(0, document.body.scrollHeight)')

    # 模拟下滑至底部
    async def scroll(self, task_index=0):
        pager = self.getTaskPagers()[task_index]
        await pager.evaluate('_ => {window.scrollBy(0, window.innerHeight);}')

    # 获取元素属性或文本
    async def getWaitElementValues(self, xpath, task_index=0, attribute=None, minnum=1, timeout=20):
        pager = self.getTaskPagers()[task_index]
        elements = []
        for i in range(timeout):
            elements = await pager.xpath(xpath)
            if len(elements) >= minnum:
                break
            else:
                await asyncio.sleep(1)

        assert len(elements) >= minnum, '[超时] %s' % xpath
        if attribute is None:
            return elements
        elif attribute == 'text':
            attribute = 'textContent'
        attributes = list()
        for element in elements:
            # 获取属性值
            attributes.append(await (await element.getProperty(attribute)).jsonValue())

        return attributes

    '''js_str可以为tag、#id.class、.class、#id等形式'''

    # 触发输入事件
    async def keyInput(self, js_str, value, task_index=0, delay=100):
        pager = self.getTaskPagers()[task_index]
        await pager.type(js_str, value, {'delay': delay})

    # 触发点击事件
    async def click(self, js_str, task_index=0):
        pager = self.getTaskPagers()[task_index]
        # 浏览器不能处于最小化，否则会一直等待
        await pager.click(js_str)

    # 模拟滑块拖动
    async def slideDrag(self, js_str, x, task_index=0, delay=1500):
        pager = self.getTaskPagers()[task_index]
        await pager.hover(js_str)  # 不同场景的验证码模块能名字不同。
        await pager.mouse.down()
        await pager.mouse.move(x, 0, {'delay': delay})
        await pager.mouse.up()

    # 聚焦元素，便于直接调用鼠标或键盘操作
    async def focus(self, js_str, task_index=0):
        pager = self.getTaskPagers()[task_index]
        await pager.focus(js_str)



import time
from appium.webdriver import WebElement
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from appium import webdriver


class Appium(webdriver.Remote):
    def __init__(self, app_package, appactivity,
                 pversion='6.0.1', device_name='driver_name',
                 allyes_authority=True, no_reset=False,
                 link="http://localhost:4723/wd/hub", **kwargs):
        self.caps = {
            "platformVersion": pversion,
            "platformName": 'Android',
            "deviceName": device_name,
            "appPackage": app_package,
            "appActivity": appactivity,
            "autoGrantPermissions": allyes_authority,
            "noReset": no_reset,
            **kwargs
        }
        webdriver.Remote.__init__(self, link, self.caps)

    def getWaitElement(self, constraint, constraint_class='id', timeout=15) -> WebElement:
        if constraint_class == 'xpath':
            constraint_class = By.XPATH
        elif constraint_class == 'id':
            constraint_class = By.ID
        elif constraint_class == 'class':
            constraint_class = By.CLASS_NAME
        elif constraint_class == 'tag':
            constraint_class = By.TAG_NAME
        else:
            assert False, 'constraint_class 约束类型出错，该元素等待方法无效！'
        e = WebDriverWait(self, timeout).until(EC.presence_of_element_located((constraint_class, constraint)))
        return e

    def getWaitElements(self, constraint, constraint_class='id', minnum=1, timeout=15, onetime=0.5) -> list:
        if constraint_class == 'xpath':
            constraint_class = By.XPATH
        elif constraint_class == 'id':
            constraint_class = By.ID
        elif constraint_class == 'class':
            constraint_class = By.CLASS_NAME
        elif constraint_class == 'tag':
            constraint_class = By.TAG_NAME
        else:
            assert False, 'constraint_class 约束类型出错，该元素等待方法无效！'
        es = []
        nowtimeout = timeout
        while len(es) < minnum:
            assert nowtimeout >= 0, '等待时间过长，仅发现%s个元素' % len(es)
            time.sleep(onetime)
            es = WebDriverWait(self, timeout, onetime).until(
                EC.presence_of_all_elements_located((constraint_class, constraint)))
            nowtimeout -= onetime
        return es



import requests
from .__superclass__ import Spider


class Session(Spider):
    def __init__(self, **kwargs):
        Spider.__init__(self, **kwargs)
        s = requests.session()
        c = requests.cookies.RequestsCookieJar()
        for i in kwargs.get('driver_cookies', []):  # 添加cookie到CookieJar
            c.set(i["name"], i["value"])
        s.cookies.update(c)  # 更新session里的cookie
        self.spider = s
        self.proxies_func = lambda p: {'https': f'http://{p}', 'http': f'http://{p}'}

    def get(self, url, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            html = self.spider.get(url, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            html = self.spider.get(url, **kwargs)
        # 保存源代码
        if savepath is not None: self.save(url, html.text, savepath)
        # 原始对象
        if ifobj:
            return html
        else:
            txt = html.text
            if ifjson:
                return json.loads(txt)
            else:
                return txt

    def post(self, url, data=None, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies is None:
            html = self.spider.post(url, data, **kwargs)
        else:
            kwargs['proxies'] = self.proxies_func(proxies)
            html = self.spider.post(url, data, **kwargs)
        # 保存源代码
        if savepath is not None: self.save(url, html.text, savepath)
        # 原始对象
        if ifobj:
            return html
        else:
            txt = html.text
            if ifjson:
                return json.loads(txt)
            else:
                return txt



import httplib2
from httplib2 import socks
from .__superclass__ import Spider
import re

httplib2.debuglevel = -1


class Httplib2(Spider):
    def __init__(self, **kwargs):
        Spider.__init__(self, **kwargs)
        self.spider = self.__getH__(self.proxies)

    def __getH__(self, proxies):
        self.proxies = proxies
        if proxies is None:
            h = httplib2.Http(timeout=self.timeout)
        else:
            ip, port = self.proxies.split(':')
            h = httplib2.Http(proxy_info=httplib2.ProxyInfo(socks.PROXY_TYPE_HTTP, ip, int(port)), timeout=self.timeout)
        return h

    def get(self, url, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if proxies == self.proxies:
            response, content = self.spider.request(url, method='GET', headers=kwargs['headers'])
        else:
            self.close()
            self.spider = self.__getH__(proxies)
            response, content = self.spider.request(url, method='GET', headers=kwargs['headers'])
        # 保存源代码
        if savepath is not None: self.save(url, content.decode('utf-8'), savepath)
        if ifobj:
            return response
        else:
            try:
                bm = re.findall('charset=([0-9a-zA-Z-]+)', response.get('content-type'))[0]
            except:
                bm = 'utf-8'
            txt = content.decode(bm)
            if ifjson:
                return json.loads(txt)
            else:
                return txt

    def post(self, url, data=None, ifobj=False, savepath=None, ifjson=False, **kwargs):
        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        kwargs['headers'] = kwargs.get('headers', self.headers)
        proxies = kwargs.get('proxies', self.proxies)
        if type(data) == dict:
            data = '&'.join([f'{d}={data[d]}' for d in data.keys()])
        if proxies == self.proxies:
            response, content = self.spider.request(url, method='POST', body=data, headers=kwargs['headers'])
        else:
            self.close()
            self.spider = self.__getH__(proxies)
            response, content = self.spider.request(url, method='POST', body=data, headers=kwargs['headers'])
        # 保存源代码
        if savepath is not None: self.save(url, content.decode('utf-8'), savepath)
        if ifobj:
            return response
        else:
            try:
                bm = re.findall('charset=([0-9a-zA-Z-]+)', response.get('content-type'))[0]
            except:
                bm = 'utf-8'
            txt = content.decode(bm)
            if ifjson:
                return json.loads(txt)
            else:
                return txt



from .base import Body
import asyncio


class Req(Body):

    # 发送get请求
    async def request_get(self, url, task_index=0):
        pager = self.getTaskPagers()[task_index]
        await pager.evaluate("""
            var xhr = new XMLHttpRequest();
            xhr.open('GET', '%s', true);
            window.text=-1;
            xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xhr.onload = function () {
                window.text= this.responseText;
            };
            xhr.send();
        """ % url)

    # 发送post请求
    async def request_post(self, url, data, task_index=0):
        pager = self.getTaskPagers()[task_index]
        if type(data) == dict:
            data = '&'.join(["{key}={value}".format(key=key, value=data[key]) for key in data.keys()])
        await pager.evaluate("""
            var xhr = new XMLHttpRequest();
            xhr.open('POST', '%s', true);
            window.text=-1;
            xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xhr.onload = function () {
                window.text= this.responseText;
            };
            xhr.send(%s);
        """ % (url, data))

    # 等待获取发送后的返回值
    async def wait_getResponse(self, maxci=10, mintime=0.5, task_index=0):
        pager = self.getTaskPagers()[task_index]
        for i in range(maxci):
            await asyncio.sleep(mintime)
            text = await pager.evaluate("return window.text;")
            if text is None:
                print("没有进行发送请求，无返回值")
                return ""
            if text != -1: return text
        print('等待时间已过，没有获取到返回值...')
        return ""



from threading import Lock
import re
import time
import traceback
import requests

# 互斥锁
mutex = Lock()
API = \
    'http://ip.ipjldl.com/api/entry?method=proxyServer.ipinfolist&packid=0&fa=0&fetch_key=&time=2&quantity=1&province=&city=&anonymous=1&ms=1&service=0&protocol=1&distinct=true&format=txt&separator=1&separator_txt='
ADDIP = 'http://www.ipjldl.com/Users-whiteIpAddNew.html?appid=716&appkey=9481418cc19467d2f0f2bd2cfb515376&whiteip='
DELAPI = 'http://www.ipjldl.com/Users-whiteIpDelNew.html?appid=716&appkey=9481418cc19467d2f0f2bd2cfb515376&whiteip=all'


def __getHtml_get(url, **args):
    CRO_headers = {
        'user-agent':
            'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36'}
    headers = args.get('headers', CRO_headers)
    timeout = args.get('timeout', 20)
    proxies = args.get('proxies', None)
    valuetype = args.get('valuetype', 'text')
    ifcheckstatus = args.get('ifcheckstatus', True)
    verify = args.get('verify', True)
    if headers != CRO_headers:
        if 'user-agent' not in headers.keys() and 'User-Agent' not in headers.keys():
            headers['user-agent'] = CRO_headers['user-agent']
    if proxies is None:
        r = requests.get(url, headers=headers, timeout=timeout, verify=verify)
    else:
        r = requests.get(url, headers=headers, proxies={'https': proxies, 'http': proxies},
                         timeout=timeout, verify=verify)
    if ifcheckstatus: r.raise_for_status()  # 如果状态不为200，返回异常
    # 原始对象
    if valuetype == 'text':
        return r.text
    elif valuetype == 'content':
        return r.content
    elif valuetype == 'json':
        return r.json()
    else:
        return r


# 未完成多个获取
def getIP():
    mutex.acquire()  # 上锁 注意了此时锁的代码越少越好
    time.sleep(1.5)
    try:
        ip_txt = __getHtml_get(API).strip()
    except:
        traceback.print_exc()
        mutex.release()  # 解锁
        ip_txt = None
    if re.match('^\d+\.\d+\.\d+\.\d+', ip_txt) is None:
        # 添加白名单
        my_ip = re.findall('(\d+\.\d+\.\d+\.\d+)登录IP不是白名单IP，请在用户中心添加该白名单', ip_txt)
        if len(my_ip) > 0:
            print('[添加白名单]', my_ip[0])
            addIP(my_ip[0])
            time.sleep(5)
            ip_txt = __getHtml_get(API).strip()
            if re.match('^\d+\.\d+\.\d+\.\d+', ip_txt) is not None:
                # 清空白名单
                print('[清空白名单]')
                clearIP()
                time.sleep(3)
                addIP(my_ip[0])
                time.sleep(5)
                ip_txt = __getHtml_get(API).strip()
                if re.match('^\d+\.\d+\.\d+\.\d+', ip_txt) is None:
                    print('[代理ip未获取]', ip_txt)
                    ip_txt = None
                    print('获取代理ip失败！ 默认使用本机ip')
        else:
            print('[代理ip未获取]', ip_txt)
            ip_txt = None
            print('获取代理ip失败！ 默认使用本机ip')
    if ip_txt is not None: print('[代理ip]', ip_txt)
    mutex.release()  # 解锁
    return ip_txt


# 添加白名单
def addIP(ip):
    __getHtml_get(ADDIP + ip)


# 清空白名单
def clearIP():
    __getHtml_get(DELAPI)


if __name__ == '__main__':
    print(getIP())



from selenium.common.exceptions import TimeoutException


# 删除所有元素后重新打开连接
# 此方法的新链接navigator为false
def newGet(driver, url):
    if driver.getnum < 1: assert False, '已达到规定的最大请求数!建议关闭后重新开启'
    try:
        driver.execute_script('document.getElementsByTagName("html")[0].remove()')
    except:
        pass
    try:
        driver.get(url)
    except TimeoutException:  # 捕获超时异常
        print("已超时,强制跳出...")
    driver.getnum -= 1


# 新建多个窗口,并打开对应链接,最多20个
def neWindows(self, urls):
    assert 0 < len(urls) <= 20, '链接数最多有20个'
    # 计算需要新开的窗口数量
    newlen = max(len(urls) - len(self.window_handles), 0)
    # 在原窗口基础上在新建n个窗口
    for i in range(newlen):
        self.execute_script('window.open()')
        hands = self.window_handles
        self.switch_to.window(hands[-1])
        self.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
            "source": """
                Object.defineProperty(navigator, 'webdriver', {
                  get: () => false})
              """})
    # 依次打开链接
    hands = self.window_handles
    for i in range(len(hands)):
        self.switch_to.window(hands[i])
        self.newGet(urls[i])


# 不等待多开
def neWindowGet(self, url):
    h1 = self.window_handles
    self.execute_script(f'window.open(\'{url}\')')
    h2 = self.window_handles
    h = (set(h2) - set(h1)).pop()
    self.close()
    self.switch_to.window(h)


# 关闭其他窗口
def closeOtherWindow(self):
    for hand in self.window_handles[1:]:
        self.switch_to.window(hand)
        self.close()
    self.switch_to.window(self.window_handles[0])


# 切换窗口
def switchWindow(self, index):
    self.switch_to.window(self.window_handles[index])


# 发送get请求
def request_get(driver, url):
    driver.execute_script("""
           var xhr = new XMLHttpRequest();
           xhr.open('GET', '%s', true);
           window.text=-1;
           xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
           xhr.onload = function () {
               window.text= this.responseText;
           };  xhr.send();""" % url)


# 发送post请求
def request_post(driver, url, data):
    if type(data) == dict:
        data = '&'.join(["{key}={value}".format(key=key, value=data[key]) for key in data.keys()])
    driver.execute_script("""
           var xhr = new XMLHttpRequest();
           xhr.open('POST', '%s', true);
           window.text=-1;
           xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
           xhr.onload = function () {
               window.text= this.responseText;
           };xhr.send(%s); """ % (url, data))



from math import inf
from .__selenium__.init import *
from .__selenium__.open import *
from .__selenium__.wait import *
from .__selenium__.operation import *
from .__selenium__.get import *
from selenium.webdriver.common.keys import Keys


# 使用相对导入的py文件不能作为启动文件
class ChromeDriver(webdriver.Chrome):
    # 默认代理文本:  代理方式://ip:porn
    # socks5://192.168.3.2:1080
    def __init__(self, ifchromium=False, custom_cd_cm: tuple = None, proxies=None, min_window=False,
                 max_window=False, headless=False, minload=False, ifiphone=False, timeout=15,
                 datapath=None, user_agent=None, url=None, getnum=inf,
                 chrome_version='90', **kwargs):
        # 初始化
        chrome_init(self, ifchromium=ifchromium, proxies=proxies, min_window=min_window, max_window=max_window,
                    headless=headless, minload=minload, ifiphone=ifiphone, datapath=datapath, custom_cd_cm=custom_cd_cm,
                    user_agent=user_agent, url=url, getnum=getnum, chrome_version=chrome_version, timeout=timeout)

    def newGet(self, url):
        return newGet(self, url)

    def neWindowGet(self, url):
        return neWindowGet(self, url)

    def addCookies(self, *cookies, iftz=True):
        return addCookies(self, cookies, iftz)

    def request_get(self, url):
        return request_get(self, url)

    def request_post(self, url, data):
        return request_post(self, url, data)

    def getWaitElement(self, constraint, timeout=10, ifassert=True):
        return getWaitElement(self, constraint, timeout=timeout, ifassert=ifassert)

    def getWaitElements(self, constraint, minnum=1, timeout=15):
        return getWaitElements(self, constraint, minnum=minnum, timeout=timeout)

    def runWaitElement(self, xpath, func, *args, **kwargs):
        return runWaitElement(self, xpath, func, *args, **kwargs)

    def getUserAgent(self):
        return getUserAgent(self)

    # 模拟清空
    def keyClear(self, xpath, timeout=15):
        e = self.getWaitElement(xpath, timeout=timeout)
        return keyClear(e)

    # 模拟输入
    def keyin(self, xpath, txt, timeout=15, ifclear=False, backspace_num=0):
        e = self.getWaitElement(xpath, timeout=timeout)
        act = ActionChains(self)
        act.move_to_element(e).click().perform()
        if ifclear:
            if backspace_num > 0:
                [self.keyin(xpath, Keys.BACK_SPACE) for i in range(backspace_num)]
            else:
                self.keyClear(xpath)
        return keyin(e, txt)

    # 模拟点击,直接调用js,需要使用唯一xapth,否则只取第一个元素
    def click(self, xpath, timeout=15):
        e = self.getWaitElement(xpath, timeout=timeout)
        act = ActionChains(self)
        act.move_to_element(e).click().perform()
        return  # click(self, xpath)

    # 选择下拉栏选项
    def select(self, xpath, value_text):
        e = self.getWaitElement(xpath)
        return select(e, value_text)

    # 模拟浏览器下滑至底部
    def scroll(self, maxwaitime=15, speed=1000):
        return scroll(self, maxwaitime=maxwaitime, speed=speed)

    # 跳至浏览器底部
    def goDown(self):
        return goDown(self)

    # 跳至浏览器顶部
    def goUp(self):
        return goUp(self)

    # 移除所有指定标签
    def removeAllTags(self, *tagnames):
        return removeAllTags(self, *tagnames)

    def slideDragAction(self, button_xpath, x):
        e = self.getWaitElement(button_xpath)
        return slideDragAction(e, x)

    # 跳转嵌套页面
    def changeFrame(self, xpath='//iframe'):
        e = self.getWaitElement(xpath)
        return changeFrame(self, e)

    # 跳转至最外层页面
    def parentFrame(self):
        return parentFrame(self)

    def closeAll(self):
        # 可能涉及到cookie问题需要调用close方法才行
        self.close()
        # time.sleep(1)
        self.quit()

    def getLocalStorage(self):
        return getLocalStorage(self)

    # 获取元素坐标
    def getElementPosition(self, xpath) -> tuple:
        self.getWaitElement(xpath)
        return getElementPosition(self, xpath)

    def getCookie_str(self) -> str:
        return getCookie_str(self)

    def switchWindow(self, index):
        return switchWindow(self, index)

    def closeOtherWindow(self):
        return closeOtherWindow(self)


# 使用相对导入的py文件不能作为启动文件
class FirefoxDriver(webdriver.Firefox):
    # 默认代理文本:  代理方式://ip:porn
    # socks5://192.168.3.2:1080
    def __init__(self, proxies=None, min_window=False,
                 max_window=False, headless=False, minload=False, ifiphone=False,
                 datapath=None, user_agent=None, url=None, getnum=inf, **kwargs):
        # 初始化
        firefox_init(self, proxies=proxies, min_window=min_window, max_window=max_window,
                     headless=headless, minload=minload, ifiphone=ifiphone, datapath=datapath,
                     user_agent=user_agent, url=url, getnum=getnum)

    def newGet(self, url): return newGet(self, url)

    def addCookies(self, *cookies, iftz=True): return addCookies(self, cookies, iftz)

    def request_get(self, url): return request_get(self, url)

    def request_post(self, url, data): return request_post(self, url, data)

    def getWaitElement(self, constraint, timeout=10, ifassert=True):
        return getWaitElement(self, constraint, timeout=timeout, ifassert=ifassert)

    def getWaitElements(self, constraint, minnum=1, timeout=15):
        return getWaitElements(self, constraint, minnum=minnum, timeout=timeout)

    def runWaitElement(self, xpath, func, *args, **kwargs):
        return runWaitElement(self, xpath, func, *args, **kwargs)

    def getUserAgent(self): return getUserAgent(self)

    # 模拟清空
    def keyClear(self, xpath):
        e = self.getWaitElement(xpath)
        return keyClear(e)

    # 模拟输入
    def keyin(self, xpath, txt):
        e = self.getWaitElement(xpath)
        return keyin(e, txt)

    # 模拟点击,直接调用js,需要使用唯一xapth,否则只取第一个元素
    def click(self, xpath): return click(self, xpath)

    # 选择下拉栏选项
    def select(self, xpath, value_text):
        e = self.getWaitElement(xpath)
        return select(e, value_text)

    # 模拟浏览器下滑至底部
    def scroll(self, maxwaitime=30, speed=500): return scroll(self, maxwaitime=maxwaitime, speed=speed)

    # 跳至浏览器底部
    def goDown(self): return goDown(self)

    # 跳至浏览器顶部
    def goUp(self): return goUp(self)

    # 移除所有指定标签
    def removeAllTags(self, *tagnames): return removeAllTags(self, *tagnames)

    def slideDragAction(self, button_xpath, x):
        e = self.getWaitElement(button_xpath)
        return slideDragAction(e, x)

    # 跳转嵌套页面
    def changeFrame(self, xpath='//iframe'):
        e = self.getWaitElement(xpath)
        return changeFrame(self, e)

    # 跳转至最外层页面
    def parentFrame(self): return parentFrame(self)



# 获取头文件信息
def getUserAgent(driver):
    return driver.execute_script("return navigator.userAgent")


# 判断页面内容是否为空
def ifNull(driver):
    txt = driver.page_source.strip()
    return txt == '<html><head></head><body></body></html>' or len(txt) == 0


# 获取LocalStorage字典
def getLocalStorage(driver) -> dict:
    return driver.execute_script('''
    var ls = window.localStorage, items = {}; 
    for (var i = 0, k; i < ls.length; ++i) 
      items[k = ls.key(i)] = ls.getItem(k); 
    return items; ''')


# 获取元素坐标
def getElementPosition(driver, xpath) -> tuple:
    dt = driver.execute_script(
        f'return document.evaluate("{xpath}", document, null, XPathResult.ANY_TYPE, null).iterateNext().getBoundingClientRect();')
    return (dt['x'], dt['y'])

def getCookie_str(driver)->str:
    return driver.execute_script('return document.cookie;')